using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using RSG;
using Newtonsoft.Json;

namespace cli.rpc.wiki.search
{
    class CdBridgeServer
    {
        public dBridges.dBridges dbridge;

        private string dbApplnKey = 'appkey';
        private string dbAuthURL = 'auth_url';
        private string searchString="";

        public void UpdatesearchString(string value)
        {
            this.searchString = value;
        }

        public async Task init_dataBridges()
        {
            // Initialize dataBridges client
            this.dbridge = new dBridges.dBridges();

            // Replace your authentication url below, which you have received from dataBridges management portal.
            this.dbridge.auth_url = this.dbAuthURL;

            // Replace your application key below, which you have received from dataBridges management portal.
            this.dbridge.appkey = this.dbApplnKey;

            //Bind to connected event, Once the dataBridges is connected successfully, Subscription and other activities can be performed.
            Action<object> connected_callback;
            this.dbridge.connectionstate.bind("connected", connected_callback = async (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->connected" + "sessionid-> " + this.dbridge.sessionid);

                dBridges.remoteprocedure.CrpCaller wiki = null;
                // Connect to rpcServer
                try {
                    wiki = await dbridge.rpc.connect("wiki.search");
                } catch (dBridges.exceptions.dBError err) {
                    // Catch connect exception
                    Console.WriteLine("dBridge rpc.connect exception {0} {1} {2}", err.source, err.code, err.message);
                }

                // Bind to event server.connect.success, This will be triggered when the connection to server is success
                Action<object, object> connetsuccess;
                wiki.bind("dbridges:rpc.server.connect.success", connetsuccess = async (payload, metadata) => {
                    // Execute Server function with parameter .
                    IPromise<object> p = await wiki.call("summary", this.searchString, 1000 * 60 * 2, null);
                    p.Then((response) => {
                        Console.WriteLine("{1}", response);
                    })
                    .Catch((err) => {
                        dBridges.exceptions.dBError er = err as dBridges.exceptions.dBError;
                        Console.WriteLine("wiki => exception {0} , {1} , {2}",  er.source, er.code, er.message);
                    });
                });

                // Bind to server.connect.fail. This will be triggered either if connection to rpcServer fails or reconnect fails.
                Action<object, object> connetfail;
                wiki.bind("dbridges:rpc.server.connect.fail", connetfail = (payload, metadata) => {
                    dBridges.exceptions.dBError perror = payload as dBridges.exceptions.dBError;
                    Console.WriteLine("dbridges:rpc.server.connect.fail {0} {1} {2} {3}", perror.source, perror.code, perror.message, metadata);
                });
            });

            //Bind to disconnected event, to get intimation about dataBridges network disconnection.
            Action<object> disconnected_callback;
            this.dbridge.connectionstate.bind("disconnected", disconnected_callback = (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->disconnected");
            });

            // Connect to dataBridges. If any runtime error it will be caught in catch().
            try {
                await this.dbridge.connect();
                string ch = Console.ReadLine();
            } catch (Exception e) {
                Console.WriteLine(e);
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            CdBridgeServer db = new CdBridgeServer();
            db.UpdatesearchString(args[0]);
            Task.Run(db.init_dataBridges).Wait();
        }
    }
}